# coding: utf-8

from __future__ import unicode_literals
from __future__ import print_function
from __future__ import absolute_import
from __future__ import division
from django.conf import settings
from disguise.const import KEYNAME
from disguise.compat import checks


def check_env():
    idx = settings.MIDDLEWARE_CLASSES.index

    # Checks if `django.contrib.sessions` application installed
    sessions_app_installed = ('django.contrib.sessions' in
                              settings.INSTALLED_APPS)

    # Checks if `django.core.context_processors.request` added
    # into TEMPLATE_CONTEXT_PROCESSORS tuple
    context_processors = ('django.core.context_processors.request' in
                          settings.TEMPLATE_CONTEXT_PROCESSORS)

    # Checks if sessions middleware is added into MIDDLEWARE_CLASSES
    sessions_middleware_installed = (
        'django.contrib.sessions.middleware.SessionMiddleware'
        in settings.MIDDLEWARE_CLASSES
    )

    # Checks if disguise middleware is added into MIDDLEWARE_CLASSES
    disguise_middleware_installed = ('disguise.middleware.DisguiseMiddleware'
                                     in settings.MIDDLEWARE_CLASSES)

    # Checks if sessions middleware installed before disguise one
    try:
        middleware_order = (
            idx('django.contrib.sessions.middleware.SessionMiddleware') <
            idx('disguise.middleware.DisguiseMiddleware')
        )
    except ValueError:
        middleware_order = False

    errors = []

    if not sessions_app_installed:
        errors.append(
            checks.Error(
                'Django sessions app is *not* installed',
                id='disguise.E001',
                hint=('Add the `django.contrib.sessions` into '
                      'your INSTALLED_APPS setting')
            )
        )
    if not context_processors:
        errors.append(
            checks.Error(
                'There is no `request` variable in template context',
                id='disguise.E002',
                hint=('Add the django.core.context_processors.request into '
                      'your TEMPLATE_CONTEXT_PROCESSORS setting')
            )
        )
    if not sessions_middleware_installed:
        errors.append(
            checks.Error(
                'Session middleware is *NOT* installed',
                id='disguise.E003',
                hint=('Add `django.contrib.sessions.middleware.Session'
                      'Middleware` into your MIDDLEWARE_CLASSES setting')
            )
        )
    if not disguise_middleware_installed:
        errors.append(
            checks.Error(
                'Disguise middleware is *NOT* installed',
                id='disguise.E004',
                hint=('Add `disguise.middleware.DisguiseMiddleware` '
                      'into your MIDDLEWARE_CLASSES setting')
            )
        )
    if not middleware_order:
        errors.append(
            checks.Error(
                'Session middleware must be installed before disguise one',
                id='disguise.E005',
                hint=('Swap the disguise.middleware.DisguiseMiddleware with '
                      'django.contrib.sessions.middleware.SessionMiddeware')
            )
        )

    return errors


def can_disguise(request):
    if KEYNAME in request.session:
        return True

    if hasattr(request, 'original_user'):
        return True

    if request.user.has_perm('disguise.can_disguise'):
        return True

    return False



# coding: utf-8

from __future__ import unicode_literals
from __future__ import print_function
from __future__ import absolute_import
from __future__ import division
from django.contrib.auth import SESSION_KEY, BACKEND_SESSION_KEY

KEYNAME = 'django_disguise:original_user'
TAGNAME = 'body'
BACKEND = 'django.contrib.auth.backends.ModelBackend'

__all__ = ['KEYNAME', 'TAGNAME', 'SESSION_KEY', 'BACKEND_SESSION_KEY',
           'BACKEND']



# coding: utf-8

from __future__ import unicode_literals
from __future__ import print_function
from __future__ import absolute_import
from __future__ import division
from django.contrib.auth.views import redirect_to_login
from django.shortcuts import redirect
from django.contrib.auth import login
from django.views.generic import View, FormView
from disguise.forms import DisguiseForm
from disguise.const import KEYNAME, SESSION_KEY, BACKEND_SESSION_KEY, BACKEND
from disguise.utils import can_disguise
from disguise.signals import disguise_applied, disguise_disapplied


class DisguiseMixin(object):
    def dispatch(self, request, *args, **kwargs):
        if not can_disguise(request):
            return redirect_to_login(request)
        return super(DisguiseMixin, self).dispatch(request, *args, **kwargs)

    def get_http_referer(self):
        return self.request.META.get('HTTP_REFERER', '/')

    def test_cookie(self):
        if self.request.session.test_cookie_worked():
            self.request.session.delete_test_cookie()


class MaskView(DisguiseMixin, FormView):
    form_class = DisguiseForm

    def switch_user(self, user):
        self.request.session[SESSION_KEY] = user.id
        self.request.session[BACKEND_SESSION_KEY] = user.backend
        self.request.user = user

    def form_valid(self, form):
        request = self.request
        if KEYNAME not in request.session:
            request.original_user = request.user
            request.session[KEYNAME] = request.original_user.pk

        # Okay, security checks complete. Log the user in.
        new_user = form.get_user()
        new_user.backend = BACKEND

        # Change current user
        self.switch_user(new_user)
        self.test_cookie()

        disguise_applied.send(sender=new_user.__class__,
                              original_user=request.original_user,
                              new_user=new_user)
        return redirect(self.get_http_referer())


class UnmaskView(DisguiseMixin, View):
    def get(self, request, *args, **kwargs):
        if hasattr(request, 'original_user'):
            # Okay, security checks complete. Log the user in.
            old_user = request.user
            new_user = request.original_user
            new_user.backend = BACKEND

            login(request, new_user)

            disguise_disapplied.send(sender=new_user.__class__,
                                     original_user=request.original_user,
                                     old_user=old_user)
        self.test_cookie()
        return redirect(self.get_http_referer())



# coding: utf-8

from __future__ import unicode_literals
from __future__ import print_function
from __future__ import absolute_import
from __future__ import division
from django import template
from disguise.forms import DisguiseForm
from disguise.utils import can_disguise


register = template.Library()


@register.inclusion_tag('disguise/form.html', takes_context=True)
def disguise_widget(context):
    request = context['request']
    context.update({
        'can_disguise': can_disguise(request),
        'form': DisguiseForm(),
        'original_user': getattr(request, 'original_user', None),
        'disguise_user': request.user,
    })
    return context



# coding: utf-8

from __future__ import unicode_literals
from __future__ import print_function
from __future__ import absolute_import
from __future__ import division
from django import forms
from django.utils.translation import ugettext_lazy as _
from disguise.compat import get_user_model


User = get_user_model()


class DisguiseForm(forms.Form):
    """
    Disguise form
    """
    username = forms.CharField(label=_('User name'), required=False)
    user_id = forms.IntegerField(label=_('User ID'), required=False)

    def clean_username(self):
        """
        Cleans username field
        """
        username = self.cleaned_data.get('username')

        if not username:
            return None

        qset = User.objects.filter(username=username)

        if qset.exists():
            return qset.get()
        raise forms.ValidationError(_('No such username'))

    def clean_user_id(self):
        """
        Cleans 'user_id' field
        """
        user_id = self.cleaned_data.get('user_id')

        if not user_id:
            return None
        qset = User.objects.filter(pk=user_id)

        if qset.exists():
            return qset.get()
        raise forms.ValidationError(_('No such user id'))

    def clean(self):
        """
        Clears whole form totally
        """
        cleaned_data = getattr(self, 'cleaned_data', {})

        if not cleaned_data.get('user_id') and \
                not cleaned_data.get('username'):
            raise forms.ValidationError(
                _('Please enter either username or user id')
            )
        return self.cleaned_data

    def get_user(self):
        """
        Returns selected user object
        """
        assert self.is_valid()

        for field in ['username', 'user_id']:
            user = self.cleaned_data.get(field)

            if not isinstance(user, User):
                continue
            return user



# coding: utf-8

from __future__ import unicode_literals
from __future__ import print_function
from __future__ import absolute_import
from __future__ import division
from disguise.compat import get_user_model
from disguise.utils import can_disguise
from disguise.const import KEYNAME


class DisguiseMiddleware(object):
    """
    Disguise user middleware
    """

    def get_original_user(self, request):
        if KEYNAME in request.session:
            return get_user_model().objects.get(pk=request.session[KEYNAME])
        return request.user

    def process_request(self, request):
        """
        Injects the `original_user` attribute into HttpRequest object
        """
        if not request.user.is_authenticated():
            return
        if not can_disguise(request):
            return
        request.original_user = self.get_original_user(request)
